<copyright> tomgnome gnome-canvas example.
    A rewrite in TOM of the Canvas example in the gnome-dev-info.
    By <a href="mailto:tiggr@gerbil.org">Pieter Schoenmakers</a>

    Copyright &copy; 1999 Pieter J. Schoenmakers

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: CanvasExample.t,v 1.3 1999/04/27 20:18:50 tiggr Exp $</id>
    </copyright>

<c>
#include <libgnomeui/libgnomeui.h>
</c>

implementation class
CanvasExample: gtomk.Constants
{
  const CANVAS_SIZE = 300.0;

  <doc> Random feed.  </doc>
  static Random random;

  <doc> Position of mouse while dragging.  </doc>
  static int lastx, lasty;

  <doc> Colors.  </doc>
  static Indexed random_colors;
}

int
  main Array args
{
  [Gnome init "tom-gnome-canvas" args [ObjectArray new]];

  /* Initialize some global variables.  */
  random = [MinimalRandom new];
  random_colors = [ObjectArray with ("red",
				     "yellow",
				     "green",
				     "cyan",
				     "blue",
				     "magenta")];

  GtkWindow window = [GtkWindow new GTK_WINDOW_TOPLEVEL];
  GtkVBox vbox = [GtkVBox new (FALSE, 0)];
  [window add vbox];
  [vbox show];

  GtkFrame frame = [GtkFrame new nil];
  [frame set_shadow_type GTK_SHADOW_IN];
  [vbox pack_start (frame, TRUE, TRUE, 0)];
  [frame show];

  GnomeCanvas canvas = [GnomeCanvas new];
  [canvas set_usize (int (CANVAS_SIZE), int (CANVAS_SIZE))];
  [canvas set_scroll_region (0.0, 0.0, CANVAS_SIZE, CANVAS_SIZE)];
  [frame add canvas];
  [canvas show];

  GtkHBox hbox = [GtkHBox new (TRUE, 0)];
  [vbox pack_start (hbox, FALSE, FALSE, 0)];
  [hbox show];

  GtkButton button = [GtkButton new_with_label "Add an object"];
  [button connect "clicked"
	  to [Invocation for selector (void add_object GnomeCanvas from pointer)
			 to: self
			 with canvas]
	  using selector (void from pointer)
	  includeSender: YES];

  [hbox pack_start (button, TRUE, TRUE, 0)];
  [button show];

  button = [GtkButton new_with_label "Exit"];
  [button connect "clicked" to self using selector (void exit_clicked)];
  [hbox pack_start (button, TRUE, TRUE, 0)];
  [button show];

  [window connect "delete_event" to self
	  using selector (boolean delete_event pointer)];
  [window show];

  [gtomk main];
}

void
  add_object GnomeCanvas canvas
	from pointer i_want_an_argumentless_last_name_part_in_tom
{
  int x1 = [random next int (CANVAS_SIZE)];
  int y1 = [random next int (CANVAS_SIZE)];
  int x2 = [random next int (CANVAS_SIZE)];
  int y2 = [random next int (CANVAS_SIZE)];

  if (x1 > x2)
    (x1, x2) = (x2, x1);
  if (y1 > y2)
    (y1, y2) = (y2, y1);

  // X2 may grow beyond CANVAS_SIZE...
  // Sun Apr 18 22:39:52 1999, tiggr@gerbil.org
  if (x2 - x1 < 10)
    x2 += 10;

  if (y2 - y1 < 10)
    y2 += 10;

  pointer root = [[canvas root] gnomeCanvasGroup];
  pointer p;

  if ([random next 2] == 1)
    {
      <c>
	 p = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
				    "x1", (double) x1,
				    "y1", (double) y1,
				    "x2", (double) x2,
				    "y2", (double) y2,
				    "fill_color", "white",
				    "outline_color", "black",
				    "width_units", 1.0, NULL);
      </c>
    }
  else
    {
      <c>
	 p = gnome_canvas_item_new (root, gnome_canvas_ellipse_get_type (),
				    "x1", (double) x1,
				    "y1", (double) y1,
				    "x2", (double) x2,
				    "y2", (double) y2,
				    "fill_color", "white",
				    "outline_color", "black",
				    "width_units", 1.0, NULL);
      </c>
    }

  GnomeCanvasRE item = [GnomeCanvasRE proxy p create: YES];
  [item connect "event" to self using selector (boolean item All event pointer)
	includeSender: YES];
}

void
  exit_clicked
{
  [gtomk main_quit];
}

boolean
  delete_event pointer the_event
{
  [gtomk main_quit];
}

boolean
   item GnomeCanvasRE item
  event pointer e
{
  int et = [GdkEvent eventType e];

  if (et == GDK_BUTTON_PRESS)
    {
      int eb = [EventButton button e];
      if (eb == 1)
	{
	  (lastx, lasty) = [EventButton position e];
	  return TRUE;
	}
      if (eb == 3)
	{
	  [item destroy];
	  return TRUE;
	}
    }
  else if (et == GDK_2BUTTON_PRESS)
    {
      if ([EventButton button e] == 1)
	{
	  [self change_item_color item];
	  return TRUE;
	}
    }
  else if (et == GDK_MOTION_NOTIFY)
    {
      int bm = [EventMotion state e];
      if (bm & GDK_BUTTON1_MASK != 0)
	{
	  int x, y;
	  (x, y) = [EventMotion position e];
	  [item move (double (x - lastx), double (y - lasty))];
	  (lastx, lasty) = (x, y);
	  return TRUE;
	}
    }
  else if (et == GDK_ENTER_NOTIFY)
    {
      [item set_width_units 3.0];
      return TRUE;
    }
  else if (et == GDK_LEAVE_NOTIFY)
    {
      [item set_width_units 1.0];
      return TRUE;
    }
}

<doc> Set a random fill color for the {item}.  </doc>
void
  change_item_color GnomeCanvasRE item
{
  int r = [random next [random_colors length]];
  [item set_fill_color random_colors[r]];
}

end;

implementation instance
CanvasExample

end;
