<copyright> gtomk tabview example.
    A small TAB-delimited table viewer.
    By <a href="mailto:yacc@gerbil.org">Andreas Kostyrka</a>

    Copyright &copy; 1998 Andreas Kostyrka

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: TabView.t,v 1.6 1998/07/21 15:37:44 yacc Exp $</id>
    </copyright>

implementation class
TabView: State, Constants

int
  main MutableArray args
{
  [[TabView alloc] init MutableObjectArray(args)];
}

end;

implementation instance
TabView

{
  <doc> contain the toplevel window. </doc>
  GtkWindow topwindow;
  <doc> contain the highest level vertical box. </doc>
  GtkVBox topbox;
  <doc> contain the actually packed data area widget. </doc>
  GtkCList dataarea;
  <doc> contain the loaded filename. </doc>
  String actualfilename;
  <doc> contain the sort order column. </doc>
  int sortorder;
  <doc> TabFile of loaded data. </doc>
  TabFile file;
}

void
  destroy
{
  [gtomk main_quit];
}

void
  set_sortorder int no
{
  sortorder=no;
  [file sortOnColumn no];
  [dataarea freeze];
  [dataarea clear];

  int i,n=[file length];
  for (i = 0; i < n; i++)
    [dataarea append file[i]];
  [dataarea thaw];
  [[[stdio err] print ("sortorder:",no)] nl];
}

void 
  setfile String filename
{
  if(([filename length]>0) &&
     (filename[[filename length] - 1]!='/'))
    {
      actualfilename=filename;
      file=[[TabFile alloc] init filename];
      sortorder=-1;
      if(dataarea!=nil) 
	[dataarea destroy];
      dataarea=[self dataarea];
      [topbox pack_start(dataarea,YES,YES,0)];
    }
}

void
  process_file_open 
{
  Invocation invoke = [Invocation for selector(void setfile String) 
				  to: self]; 

  [[[FileSelectionController alloc] 
     initOk invoke]
    run];
}

GtkMenuBar 
  menubar
{
  GtkMenuBar menubar = [GtkMenuBar new];
  GtkMenu w_filemenu = [GtkMenu new];
  GtkMenuItem filemenu = [GtkMenuItem new_with_label "File"];
  [filemenu set_submenu w_filemenu];
  [filemenu show];

  GtkMenuItem item = [GtkMenuItem new_with_label "Open table..."];
  [w_filemenu append item];
  [item connect "activate" to self using selector (void process_file_open)];
  [item show];
  item = [GtkMenuItem new_with_label "Quit"];
  [w_filemenu append item];
  [item connect "activate" to self using selector (void destroy)];
  [item show];

  [menubar append filemenu];
  [menubar show];
  = menubar;
}

GtkCList (res)
  dataarea
{
  if (file==nil)
    {
      [[[stdio err] print "file==nil"] nl];
      res = Any([GtkLabel new "No table loaded."]);
      [res show];
    }
  else 
    {
      res = [GtkCList new [file columns]];
      [GtkCList(res) freeze];

      int i,n=[file columns];
      for(i = 0; i < n; i++) 
	{
	MutableByteString header=[MutableByteString new];
	[header print ("column ", i + 1)];
	[GtkCList(res) set_column_title (i,header)];
	}

      MutableIntArray widths = [MutableIntArray new];
      for (i = 0; i < n; i++)
	[widths add 0];
      GdkFont clistfont = [[res style] font];

      int j,m = [file length];

      for (j = 0; j < m; j++) {
	ObjectArray row = file[j];
	[GtkCList(res) append row];
	
	for (i = 0; i < n; i++) {
	  int maxwidth = widths[i];
	  int newwidth = [clistfont string_width row[i]];
	  if (newwidth > maxwidth) {
	    widths[i] = newwidth;
	  }
	}
      }

      for (i = 0; i < n; i++) {
	int w = widths[i];
	[GtkCList(res) set_column_width (i, w)];
	GtkCListColumn col=[GtkCList(res) column i];
	[[col button] connect "clicked"
		      to [Invocation of selector(void set_sortorder int)
				     to: self
				     with i]
	              using selector (void fire)];
	
      }

      [GtkCList(res) column_titles_show];
      
      [GtkCList(res) thaw];

      [res show];
    }
}

void
  createWidgets
{
  topwindow = [GtkWindow new GTK_WINDOW_TOPLEVEL];
  [topwindow connect "destroy" to self using selector (void destroy) after: NO];
  [topwindow set_usize (600,150)];

  topbox = [GtkVBox new (FALSE,0)];
  [topbox show];
  [topwindow add topbox];

  GtkMenuBar menu = [self menubar];
  [topbox pack_start(menu,NO,NO,0)];

  dataarea=[self dataarea];
  [topbox pack_start(dataarea,YES,YES,0)];

  [topwindow show];
}

id 
  init MutableObjectArray arguments
{
  [gtomk init arguments];
  [self createWidgets];
  if ([arguments length]==1)
    [self setfile String(arguments[0])];
  [gtomk main];
}

end;

implementation class FileSelectionController : State
end;

implementation instance FileSelectionController {
  <doc> contain our file selection dialog. </doc>
  GtkFileSelection dialog;
  <doc> is invoked if Ok is clicked. </doc>
  Invocation setfile;
}

id 
  initOk Invocation ok
{
  setfile=ok;
  = self;
}

void 
  run
{
  dialog = [GtkFileSelection new "Open table"];
  [dialog show];
  [gtomk grab_add dialog];

  [[dialog ok_button] connect "clicked" 
		      to self
		      using selector (void clicked)];
  [[dialog cancel_button] connect "clicked" 
			  to self
			  using selector (void cancel)];
}

void
  clicked
{
  [setfile fireWith [dialog get_filename]];
  [gtomk grab_remove dialog];
  [dialog destroy];

  dialog=nil;
  setfile=nil;
}

void
  cancel
{
  [gtomk grab_remove dialog];
  [dialog destroy];
  dialog=nil;
  setfile=nil;
}

end;

