<copyright> gtomk tabview example.
    A small TAB-delimited table viewer.
    By <a href="mailto:yacc@gerbil.org">Andreas Kostyrka</a>

    Copyright &copy; 1998 Andreas Kostyrka

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: tabfile.t,v 1.5 1998/07/21 15:37:46 yacc Exp $</id>
    </copyright>

implementation class TabFile : State, tom.Constants
end;

implementation instance TabFile {
  <doc> contains the data rows of the file. </doc>
  public MutableObjectArray rows;
  <doc> contains the maximal number of columns. </doc>
  public int columns;
  <doc> column to sort upon. </doc>
  public int keyno;
}

id
  init String filename
{
  File f = [File open filename input: TRUE];
  [f seek 0 relative: STREAM_SEEK_END];
  long filesize = [f position];
  [f seek 0 relative: STREAM_SEEK_SET];
  
  BufferedStream bs = [[BufferedStream alloc] 
			init f
			bufferSize 64*1024];

  MutableByteString filevalue = [[MutableByteString alloc] 
				  initWithCapacity int(filesize)];
  
  [bs readBytes int(filesize) into filevalue];

  rows=MutableObjectArray([filevalue componentsSeparatedBy '\n']);

  int i,n=[rows length];
  columns=0;
  for(i=0;i<n;i++) 
    {
      String row = rows[i];
      MutableArray splitup = [row componentsSeparatedBy '\t'];
      int l = [splitup length];
      if (l>columns)
	columns=l;
      rows[i]=splitup;
    }

  MutableObjectArray newrows = [[MutableObjectArray alloc]
				 initWithCapacity n];

  int wp=0;

  for(i=0;i<n;i++)
    {
      MutableObjectArray row = rows[i];
      int j,l = [row length];
      if(l>0) 
	{
	  RowArray newrow = [RowArray new];
	  [newrow set_owner self];
	  for (j = 0; j < columns; j++)
	    if (j < l)
	      [newrow add Any(row[j])];
	    else
	      [newrow add ""];
	  newrows[wp++] = newrow;
	}
    }

  rows=newrows;

  = self;
}

int 
  length
{
  = [rows length];
}

ObjectArray
  at int row
pre
  row<[rows length]
{
  = rows[row];
}

String
  at (int,int) (row,col)
pre
  col<columns && row<[rows length]
{
  ObjectArray datarow=rows[row];
  if(col<[datarow length])
    = datarow[col];
  else
    = "";
}

void
  sortOnColumn int no
pre
  no<columns
{
  keyno=no;
  [rows quickSortUsingKey selector(Any sortkey)];
}
end;

implementation class 
RowArray: MutableObjectArray

end;

implementation instance
RowArray
{
  <doc> Our owner. </doc>
  public mutable TabFile owner;
}

Any 
  sortkey
{
  = self[[owner keyno]];
}
end;
