<copyright> gdomk Drawable class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GDOMK.  GDOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GDOMK distribution; see the file COPYING.LIB.

    <id>$Id: Drawable.t,v 1.4 1999/04/28 23:05:16 tiggr Exp $</id>
    </copyright>

<c>
#include "glue.h"
#include <tom/util.h>
</c>

implementation class
GdkDrawable: GdkWrapper

end;

implementation instance
GdkDrawable

<doc> Return the {gdkDrawable} for which this object stands.  </doc>
pointer
  gdkDrawable
{
  = object;
}

<doc> Return a new default graphics context on the receiving drawable.  </doc>
GdkGC
  new_gc
{
  pointer gdkgc, p = object;

  <c>
    gdkgc = gdk_gc_new (p);
  </c>

  = [GdkGC proxy gdkgc create: YES];
}

void
  point (GdkGC, int, int) (gc, x, y)
{
  pointer obj = object;
  pointer cgc = [gc gdkGC];
  <c> gdk_draw_point ((GdkDrawable *) obj, (GdkGC *) cgc, x, y); </c>
}

void 
  line (GdkGC, int, int, int, int) (gc, x1, y1, x2, y2)
{
  pointer obj = object;
  pointer cgc = [gc gdkGC];
  <c> gdk_draw_line ((GdkDrawable *) obj, (GdkGC *) cgc, x1, y1, x2, y2); </c>
}

void
  rectangle (GdkGC, boolean, int, int, int, int)
		 (gc, filled, x, y, width, height)
{
  pointer obj = object;
  pointer cgc = [gc gdkGC];

<c>
  gdk_draw_rectangle (obj, cgc, filled, x, y, width, height);
</c>
}

void
  arc (GdkGC, boolean, int, int, int, int, int, int)
		 (gc, filled, x, y, width, height, angle1, angle2)
{
  pointer obj = object;
  pointer cgc = [gc gdkGC];

<c>
  gdk_draw_arc (obj, cgc, filled, x, y, width, height, angle1, angle2);
</c>
}

void
  string (GdkFont, GdkGC, int, int, String) (font, gc, x, y, str)
{
  pointer obj = object;
  pointer cfont = [font gdkFont];
  pointer cgc = [gc gdkGC];
  <c> {
    int len;
    char *cstr;
    DECL_USEL (tom, _pi__byteStringContents);

    C_STRING_WITH_TOM_STRING (cstr, len, str);
    gdk_draw_text ((GdkDrawable *) obj, (GdkFont *) cfont, (GdkGC *) cgc,
		     x, y, cstr, len);
    } </c>
}

void
  text (GdkFont, GdkGC, int, int, String) (font, gc, x, y, str)
{
  [self string (font, gc, x, y, str)];
}

/* void gdk_draw_polygon (GdkDrawable  *drawable,
//			  GdkGC	       *gc,
//			  gint		filled,
//			  GdkPoint     *points,
//			  gint		npoints);
*/

void
  pixmap (GdkGC, GdkDrawable, int, int, int, int, int, int)
	      (gc, src, xsrc, ysrc, xdest, ydest, width, height)
{
  pointer obj = object;
  pointer src_obj = [src gdkDrawable];
  pointer cgc = [gc gdkGC];
<c>
  gdk_draw_pixmap (obj, cgc, src_obj, xsrc, ysrc, xdest, ydest, width, height);
</c>
}

<unimplemented>
void
  bitmap (GdkGC, GdkDrawable, int, int, int, int, int, int)
	      (gc, src, xsrc, ysrc, xdest, ydest, width, height)
{
  pointer obj = object;
  pointer src_obj = [src gdkDrawable];
  pointer cgc = [gc gdkGC];
<c>
  gdk_draw_bitmap (obj, cgc, src_obj, xsrc, ysrc, xdest, ydest, width, height);
</c>
}
</unimplemented>

void
  image (GdkGC, GdkImage, int, int, int, int, int, int)
	      (gc, image, xsrc, ysrc, xdest, ydest, width, height)
{
  pointer obj = object;
  pointer cimage = [image gdkImage];
  pointer cgc = [gc gdkGC];
<c>
  gdk_draw_image (obj, cgc, cimage, xsrc, ysrc, xdest, ydest, width, height);
</c>
}

/*void gdk_draw_points	 (GdkDrawable  *drawable,
//			  GdkGC	       *gc,
//			  GdkPoint     *points,
//			  gint		npoints);
//void gdk_draw_segments	 (GdkDrawable  *drawable,
//			  GdkGC	       *gc,
//			  GdkSegment   *segs,
//			  gint		nsegs);
//void gdk_draw_lines      (GdkDrawable  *drawable,
//                          GdkGC        *gc,
//                          GdkPoint     *points,
//                          gint          npoints);
*/
end;
