<copyright> gdomk Event class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GDOMK.  GDOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GDOMK distribution; see the file COPYING.LIB.

    <id>$Id: Event.t,v 1.5 1999/04/25 18:39:16 tiggr Exp $</id>
    </copyright>

<c>
#include "glue.h"
</c>

implementation class
GdkEvent: GdkWrapper

<doc> Return the event type of the given GdkEvent.  </doc>
int (type)
  eventType pointer p
{
<c>
  type = ((GdkEvent *) p)->type;
</c>
}

<doc> Return {TRUE} for {event}s that are intended to be wrapped by the
    receiver.  The default implementation wraps any event and always
    returns {TRUE}.  </doc>
boolean
  validEvent pointer event
{
  = TRUE;
}

end;

implementation instance
GdkEvent

<doc> Return the event type of the receiving event.  </doc>
int
  eventType
{
  = [isa eventType object];
}

end;

/******************** EventButton ********************/

implementation class
EventButton: GdkEvent

<doc> Only allow button press and release events to be wrapped by an
    {EventExpose}.  </doc>
instance (id)
  wrap pointer object
pre
  [self validEvent object]
{
  = [super wrap object];
}

<doc> Return the button pressed in the {GdkEventButton} {event}.  </doc>
int (button)
  button pointer event
pre
  [self validEvent event]
{
<c>
  button = ((GdkEventButton *) event)->button;
</c>
}

<doc> Return the position in the {GdkEventButton} {event}.  </doc>
(int, int) (x, y)
  position pointer event
pre
  [self validEvent event]
{
<c>
  x = ((GdkEventButton *) event)->x;
  y = ((GdkEventButton *) event)->y;
</c>
}

<doc> Return {TRUE} for button press and button release events.  </doc>
boolean
  validEvent pointer event
{
  int et = [self eventType event];
  = et >= GDK_BUTTON_PRESS && et <= GDK_BUTTON_RELEASE;
}

end;

implementation instance
EventButton

int
  button
{
  = [isa button object];
}

(int, int)
  position
{
  = [isa position object];
}

end;

/******************** EventExpose ********************/

implementation class
EventExpose: GdkEvent

<doc> Wrap {GDK_EXPOSE} events.  </doc>
instance (id)
  wrap pointer object
pre
  [self validEvent object]
{
  = [super wrap object];
}

<doc> Return the area in the {GdkEventExpose} {event}.  </doc>
(int, int, int, int) (x, y, w, h)
  area pointer event
pre
  [self validEvent event]
{
<c>
  x = ((GdkEventExpose *) event)->area.x;
  y = ((GdkEventExpose *) event)->area.y;
  w = ((GdkEventExpose *) event)->area.width;
  h = ((GdkEventExpose *) event)->area.height;
</c>
}

<doc> Return {TRUE} for expose events.  </doc>
boolean
  validEvent pointer event
{
  int et = [self eventType event];
  = et == GDK_EXPOSE;
}

end;

implementation instance
EventExpose

end;

/******************** EventMotion ********************/

implementation class
EventMotion: GdkEvent

<doc> Only allow motion events to be wrapped by an {EventMotion}.  </doc>
instance (id)
  wrap pointer object
pre
  [self validEvent object]
{
  = [super wrap object];
}

boolean (is_hint)
  is_hint pointer event
pre
  [self validEvent event]
{
<c>
  is_hint = !!((GdkEventMotion *) event)->is_hint;
</c>
}

<doc> Return the position in the {GdkEventMotion} {event}.  </doc>
(int, int) (x, y)
  position pointer event
pre
  [self validEvent event]
{
<c>
  x = ((GdkEventMotion *) event)->x;
  y = ((GdkEventMotion *) event)->y;
</c>
}

<doc> Return the state in the {GdkEventMotion} {event}.  </doc>
int (state)
  state pointer event
pre
  [self validEvent event]
{
<c>
  state = ((GdkEventMotion *) event)->state;
</c>
}

<doc> Return {TRUE} for motion events.  </doc>
boolean
  validEvent pointer event
{
  int et = [self eventType event];
  = et == GDK_MOTION_NOTIFY;
}

end;

implementation instance
EventMotion

end;
