<copyright> gtomk Style class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: Style.t,v 1.7 1998/06/04 17:07:49 yacc Exp $</id>
    </copyright>

<gtk_version>1.0.0</gtk_version>

<c>
#include "glue.h"
</c>

<doc> A {GtkStyle} is not a {GtkObject}, and thus is a {GtkStyle} a
    {GdkWrapper}.  </doc>
implementation class
GtkStyle: GdkWrapper

<doc> Create a new style, using {gtk_style_new}.  </doc>
instance (id)
  new
{
  pointer p;
<c>
  p = gtk_style_new ();
</c>
  = [[self alloc] init p];
}

end;

implementation instance
GtkStyle

<doc> Return the {gtkStyle} for which this object stands.  </doc>
pointer
  gtkStyle
{
  = object;
}

<doc> Designated initializer.  Reference the underlying GtkStyle.  </doc>
id
  init pointer p
{
  = [[super init p] ref];
}

/******************** methods ********************/

<doc> Pass to {gtk_style_copy}.  </doc>
pointer (gtkstyle)
  copy
{
  pointer obj = object;
<c>
  gtkstyle = gtk_style_copy (obj);
</c>
}

<doc> High-level equivalent of the above.  </doc>
id
  copy
{
  = [isa proxy [self copy] create: YES];
}

<doc> Pass to {gtk_style_attach}.  </doc>
pointer (gtkstyle)
  attach GdkWindow window
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtkstyle = gtk_style_attach (obj, wobj);
</c>
}

<doc> High-level equivalent of the above.  </doc>
id
  attach GdkWindow window
{
  pointer p = [self attach window];
  = p == object ? self : [isa proxy p create: YES];
}

void
  detach
{
  pointer obj = object;
<c>
  gtk_style_detach (obj);
</c>
}

<doc> Reference the style using {gtk_style_ref}, and return {self}.  </doc>
id (self)
  ref
{
  pointer obj = object;
<c>
  gtk_style_ref (obj);
</c>
}

void
  gtk_style_unref
{
  pointer obj = object;
<c>
  gtk_style_unref (obj);
</c>
}

void
  dealloc
{
  [super dealloc];
  [self gtk_style_unref];
  pointer obj;
  object = obj;
}

void
  set_background (GdkWindow, int) (window, GTK_STATE_something)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_style_set_background (obj, wobj, GTK_STATE_something);
</c>
}

void
  set_foreground (int, GdkColor) (GTK_STATE_something, color)
{
  pointer obj = object, cobj = [color gdkColor];
<c>
  ((GtkStyle *) obj)->fg[GTK_STATE_something] = *((GdkColor *) cobj);
</c>
}

// This is presented as a GtkStyle class method in gtkstyle.h...  weird.
// Mon Apr  6 22:06:41 1998, tiggr@gerbil.org

void
  draw_hline (GdkWindow, int, int, int, int)
	     (window, GTK_STATE_something, x1, x2, y)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_draw_hline (obj, wobj, GTK_STATE_something, x1, x2, y);
</c>
}

void
  draw_vline (GdkWindow, int, int, int, int)
	     (window, GTK_STATE_something, y1, y2, x)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_draw_vline (obj, wobj, GTK_STATE_something, y1, y2, x);
</c>
}

void
  draw_shadow (GdkWindow, int, int, int, int, int, int)
	      (window, GTK_STATE_something, GTK_SHADOW_something,
	       x, y, width, height)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_draw_shadow (obj, wobj, GTK_STATE_something, GTK_SHADOW_something,
		   x, y, width, height);
</c>
}

//void gtk_draw_polygon (GtkStyle	     *style,
//		       GdkWindow     *window,
//		       GtkStateType   state_type,
//		       GtkShadowType  shadow_type,
//		       GdkPoint	     *points,
//		       gint	      npoints,
//		       gint	      fill);

void
  draw_arrow (GdkWindow, int, int, int, boolean, int, int, int, int)
	      (window, GTK_STATE_something, GTK_SHADOW_something,
	       GTK_ARROW_something, fill, x, y, width, height)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_draw_arrow (obj, wobj, GTK_STATE_something, GTK_SHADOW_something,
		  GTK_ARROW_something, fill, x, y, width, height);
</c>
}

void
  draw_diamond (GdkWindow, int, int, int, int, int, int)
	       (window, GTK_STATE_something, GTK_SHADOW_something,
		x, y, width, height)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_draw_diamond (obj, wobj, GTK_STATE_something, GTK_SHADOW_something,
		    x, y, width, height);
</c>
}

void
  draw_oval (GdkWindow, int, int, int, int, int, int)
	    (window, GTK_STATE_something, GTK_SHADOW_something,
	     x, y, width, height)
{
  pointer obj = object, wobj = [window gdkWindow];
<c>
  gtk_draw_diamond (obj, wobj, GTK_STATE_something, GTK_SHADOW_something,
		    x, y, width, height);
</c>
}

void
  draw_string (GdkWindow, int, int, int, String)
	      (window, GTK_STATE_something, x, y, string)
{
  pointer obj = object, wobj = [window gdkWindow];
  int len;
<c>
  {
    char *s;
    C_STRING_WITH_TOM_STRING (s, len, string);
    gtk_draw_string (obj, wobj, GTK_STATE_something, x, y, s);
  }
</c>
}

/******************** basic accessors1: pointers ********************/

pointer (gdkcolor)
  color_fg int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->fg[state];
</c>
}

pointer (gdkcolor)
  color_bg int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->bg[state];
</c>
}

pointer (gdkcolor)
  color_light int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->light[state];
</c>
}

pointer (gdkcolor)
  color_dark int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->dark[state];
</c>
}

pointer (gdkcolor)
  color_mid int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->mid[state];
</c>
}

pointer (gdkcolor)
  color_text int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->text[state];
</c>
}

pointer (gdkcolor)
  color_base int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->base[state];
</c>
}

pointer (gdkcolor)
  color_black
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->black;
</c>
}

pointer (gdkcolor)
  color_white
{
  pointer obj = object;
<c>
  gdkcolor = &((GtkStyle *) obj)->white;
</c>
}

GdkFont (res)
  font
{
  pointer obj = object;
  pointer gdkfont;
<c>
  gdkfont = ((GtkStyle *) obj)->font;
</c>
  res = [GdkFont proxy gdkfont create: YES];
}

pointer (gdkgc)
  fg_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->fg_gc[state];
</c>
}

pointer (gdkgc)
  bg_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->bg_gc[state];
</c>
}

pointer (gdkgc)
  light_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->light_gc[state];
</c>
}

pointer (gdkgc)
  dark_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->dark_gc[state];
</c>
}

pointer (gdkgc)
  mid_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->mid_gc[state];
</c>
}

pointer (gdkgc)
  text_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->text_gc[state];
</c>
}

pointer (gdkgc)
  base_gc int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->base_gc[state];
</c>
}

pointer (gdkgc)
  black_gc
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->black_gc;
</c>
}

pointer (gdkgc)
  white_gc
{
  pointer obj = object;
<c>
  gdkgc = ((GtkStyle *) obj)->white_gc;
</c>
}

pointer (gdkpixmap)
  bg_pixmap int state
pre
  state >= 0 && state < 6
{
  pointer obj = object;
<c>
  gdkpixmap = ((GtkStyle *) obj)->bg_pixmap[state];
</c>
}

int (result)
  depth
{
  pointer obj = object;
<c>
  result = ((GtkStyle *) obj)->depth;
</c>
}

pointer (gdkcolormap)
  colormap
{
  pointer obj = object;
<c>
  gdkcolormap = ((GtkStyle *) obj)->colormap;
</c>
}

/******************** basic accessors2: objects ********************/

GdkGC
  fg_gc int state
{
  = [GdkGC proxy [self fg_gc state] create: YES];
}

GdkGC
  bg_gc int state
{
  = [GdkGC proxy [self bg_gc state] create: YES];
}

GdkGC
  light_gc int state
{
  = [GdkGC proxy [self light_gc state] create: YES];
}

GdkGC
  dark_gc int state
{
  = [GdkGC proxy [self dark_gc state] create: YES];
}

GdkGC
  mid_gc int state
{
  = [GdkGC proxy [self mid_gc state] create: YES];
}

GdkGC
  text_gc int state
{
  = [GdkGC proxy [self text_gc state] create: YES];
}

GdkGC
  base_gc int state
{
  = [GdkGC proxy [self base_gc state] create: YES];
}

GdkGC
  black_gc
{
  = [GdkGC proxy [self black_gc] create: YES];
}

GdkGC
  white_gc
{
  = [GdkGC proxy [self white_gc] create: YES];
}

GdkPixmap
  bg_pixmap int state
{
  = [GdkPixmap proxy [self bg_pixmap state] create: YES];
}

GdkColormap
  colormap
{
  = [GdkColormap proxy [self colormap] create: YES];
}

end;
