<copyright> gtomk TOMWidget class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1999 Pieter J. Schoenmakers.

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: TOMWidget.t,v 1.2 1999/04/27 20:19:37 tiggr Exp $</id>
    </copyright>

<c>
#include "glue.h"
#include "tomwidget.h"

static GtkWidgetClass *parent_class = NULL;

/* A copy of the GtkTomwidgetClass before all signals were overridden in
   gtk_tomwidget_class_init.  */
GtkTomwidgetClass tomwidget_original_methods;

static void
gtk_tomwidget_class_init (GtkTomwidgetClass *klass)
{
  GtkObjectClass *object_class = (GtkObjectClass*) klass;
  GtkWidgetClass *widget_class = (GtkWidgetClass*) klass;

  parent_class = gtk_type_class (GTK_TYPE_WIDGET);

  tomwidget_original_methods = *klass;

  object_class->set_arg = tomwidget_object_set_arg;
  object_class->get_arg = tomwidget_object_get_arg;
  object_class->shutdown = tomwidget_object_shutdown;
  object_class->destroy = tomwidget_object_destroy;
  object_class->finalize = tomwidget_object_finalize;

  widget_class->show = tomwidget_widget_show;
  widget_class->show_all = tomwidget_widget_show_all;
  widget_class->hide = tomwidget_widget_hide;
  widget_class->hide_all = tomwidget_widget_hide_all;
  widget_class->map = tomwidget_widget_map;
  widget_class->unmap = tomwidget_widget_unmap;
  widget_class->realize = tomwidget_widget_realize;
  widget_class->unrealize = tomwidget_widget_unrealize;
  widget_class->draw = tomwidget_widget_draw;
  widget_class->draw_focus = tomwidget_widget_draw_focus;
  widget_class->draw_default = tomwidget_widget_draw_default;
  widget_class->size_request = tomwidget_widget_size_request;
  widget_class->size_allocate = tomwidget_widget_size_allocate;
  widget_class->state_changed = tomwidget_widget_state_changed;
  widget_class->parent_set = tomwidget_widget_parent_set;
  widget_class->style_set = tomwidget_widget_style_set;
  widget_class->add_accelerator = tomwidget_widget_add_accelerator;
  widget_class->remove_accelerator = tomwidget_widget_remove_accelerator;
  widget_class->grab_focus = tomwidget_widget_grab_focus;
  widget_class->event = tomwidget_widget_event;
  widget_class->button_press_event = tomwidget_widget_button_press_event;
  widget_class->button_release_event = tomwidget_widget_button_release_event;
  widget_class->motion_notify_event = tomwidget_widget_motion_notify_event;
  widget_class->delete_event = tomwidget_widget_delete_event;
  widget_class->destroy_event = tomwidget_widget_destroy_event;
  widget_class->expose_event = tomwidget_widget_expose_event;
  widget_class->key_press_event = tomwidget_widget_key_press_event;
  widget_class->key_release_event = tomwidget_widget_key_release_event;
  widget_class->enter_notify_event = tomwidget_widget_enter_notify_event;
  widget_class->leave_notify_event = tomwidget_widget_leave_notify_event;
  widget_class->configure_event = tomwidget_widget_configure_event;
  widget_class->focus_in_event = tomwidget_widget_focus_in_event;
  widget_class->focus_out_event = tomwidget_widget_focus_out_event;
  widget_class->map_event = tomwidget_widget_map_event;
  widget_class->unmap_event = tomwidget_widget_unmap_event;
  widget_class->property_notify_event
    = tomwidget_widget_property_notify_event;
  widget_class->selection_clear_event
    = tomwidget_widget_selection_clear_event;
  widget_class->selection_request_event
    = tomwidget_widget_selection_request_event;
  widget_class->selection_notify_event
    = tomwidget_widget_selection_notify_event;
  widget_class->proximity_in_event = tomwidget_widget_proximity_in_event;
  widget_class->proximity_out_event = tomwidget_widget_proximity_out_event;
  widget_class->visibility_notify_event
    = tomwidget_widget_visibility_notify_event;
  widget_class->client_event = tomwidget_widget_client_event;
  widget_class->no_expose_event = tomwidget_widget_no_expose_event;
  widget_class->selection_get = tomwidget_widget_selection_get;
  widget_class->selection_received = tomwidget_widget_selection_received;
  widget_class->drag_begin = tomwidget_widget_drag_begin;
  widget_class->drag_end = tomwidget_widget_drag_end;
  widget_class->drag_data_get = tomwidget_widget_drag_data_get;
  widget_class->drag_data_delete = tomwidget_widget_drag_data_delete;
  widget_class->drag_leave = tomwidget_widget_drag_leave;
  widget_class->drag_motion = tomwidget_widget_drag_motion;
  widget_class->drag_drop = tomwidget_widget_drag_drop;
  widget_class->drag_data_received = tomwidget_widget_drag_data_received;
  widget_class->debug_msg = tomwidget_widget_debug_msg;
}

static void
gtk_tomwidget_init (GtkTomwidget *w, void *real_class)
{
  /* No need to do anything.  */
}

GtkType
gtk_tomwidget_get_type (void)
{
  static GtkType tomwidget_type = 0;

  if (!tomwidget_type)
    {
      static const GtkTypeInfo tomwidget_info =
      {
	"TOMWidget",
	sizeof (GtkTomwidget),
	sizeof (GtkTomwidgetClass),
	(GtkClassInitFunc) gtk_tomwidget_class_init,
	(GtkObjectInitFunc) gtk_tomwidget_init,
        /* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      tomwidget_type = gtk_type_unique (GTK_TYPE_WIDGET, &tomwidget_info);
    }

  return tomwidget_type;
}

GtkWidget *
gtk_tomwidget_new (void)
{
  return gtk_type_new (gtk_tomwidget_get_type ());
}

</c>

<doc> The {TOMWidget} class is the superclass for widgets that are written
    in TOM.

    To the Gtk machinery, all {TOMWidget} subclasses will appear to be of
    the same Gtk class (named Tomwidget).  Furthermore, the {TOMWidget}
    subclasses can not define additional signals; only those inherited
    from {GtkWidget} are available.

    The methods {foo} are inherited from {GtkWidget}.  The methods
    {signal_foo} are invoked as the signal function during the signal
    emission.  We apologize for this illogical setup; it is inheritent to
    the setup of Gtk (it is visible in Gtk using functions like
    {gtk_widget_show} (the one to invoke, which emits the signal) and
    {gtk_widget_real_show} (the signal function).  </doc>
implementation class
TOMWidget: GtkWidget

<doc> Designated allocator.  </doc>
instance (id)
  new
{
  pointer p;

  <c>
     p = gtk_tomwidget_new ();
  </c>

  = [self wrap p];
}

end;

implementation instance
TOMWidget

/******************** GtkObject slots ********************/

void
  signal_set_arg (pointer, int) (arg, arg_id)
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.parent_class.set_arg)
       tomwidget_original_methods.parent_class.parent_class.set_arg (p, arg, arg_id);
  </c>
}

void
  signal_get_arg (pointer, int) (arg, arg_id)
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.parent_class.get_arg)
       tomwidget_original_methods.parent_class.parent_class.get_arg (p, arg, arg_id);
  </c>
}

void
  signal_shutdown
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.parent_class.shutdown)
       tomwidget_original_methods.parent_class.parent_class.shutdown (p);
  </c>
}

<doc> This one is probably never invoked, because the auxiliary
    information in which we hold the {"tom_id"} is destroyed before this
    signal is emitted.  </doc>
void
  signal_destroy
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.parent_class.destroy)
       tomwidget_original_methods.parent_class.parent_class.destroy (p);
  </c>
}

<doc> This one is probably never invoked, because the auxiliary
    information in which we hold the {"tom_id"} is destroyed before this
    signal is emitted.  </doc>
void
  signal_finalize
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.parent_class.finalize)
       tomwidget_original_methods.parent_class.parent_class.finalize (p);
  </c>
}

/******************** GtkWidget slots ********************/

void
  signal_show
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.show)
       tomwidget_original_methods.parent_class.show (p);
  </c>
}

void
  signal_show_all
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.show_all)
       tomwidget_original_methods.parent_class.show_all (p);
  </c>
}

void
  signal_hide
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.hide)
       tomwidget_original_methods.parent_class.hide (p);
  </c>
}

void
  signal_hide_all
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.hide_all)
       tomwidget_original_methods.parent_class.hide_all (p);
  </c>
}

void
  signal_map
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.map)
       tomwidget_original_methods.parent_class.map (p);
  </c>
}

void
  signal_unmap
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.unmap)
       tomwidget_original_methods.parent_class.unmap (p);
  </c>
}

void
  signal_realize
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.realize)
       tomwidget_original_methods.parent_class.realize (p);
  </c>
}

void
  signal_unrealize
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.unrealize)
       tomwidget_original_methods.parent_class.unrealize (p);
  </c>
}

void
  signal_draw (int, int, int, int) (x, y, w, h)
{
  pointer p = object;

  <c>
    {
      GdkRectangle area = {x, y, w, h};
      if (tomwidget_original_methods.parent_class.draw)
	tomwidget_original_methods.parent_class.draw (p, &area);
    }
  </c>
}

void
  signal_draw_focus
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.draw_focus)
       tomwidget_original_methods.parent_class.draw_focus (p);
  </c>
}

void
  signal_draw_default
{
  pointer p = object;

  <c>
     if (tomwidget_original_methods.parent_class.draw_default)
       tomwidget_original_methods.parent_class.draw_default (p);
  </c>
}

(int, int) (w, h)
  signal_size_request
{
  pointer p = object;

  <c>
    {
      GtkRequisition req;
      if (tomwidget_original_methods.parent_class.size_request)
	{
	  tomwidget_original_methods.parent_class.size_request (p, &req);
	  w = req.width, h = req.height;
	}
    }
  </c>
}

void
  signal_size_allocate (int, int, int, int) (x, y, w, h)
{
  pointer p = object;

  <c>
    {
      GtkAllocation alloc = {x, y, w, h};
      if (tomwidget_original_methods.parent_class.size_allocate)
	tomwidget_original_methods.parent_class.size_allocate (p, &alloc);
    }
  </c>
}

void
  signal_state_changed int gtk_state_previous_state
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.state_changed)
	tomwidget_original_methods.parent_class.state_changed
	  (p, gtk_state_previous_state);
    }
  </c>
}

void
  signal_parent_set GtkWidget previous_parent
{
  pointer p = object, pp;

  if (previous_parent != nil)
    pp = [previous_parent gtkWidget];

  <c>
    {
      if (tomwidget_original_methods.parent_class.parent_set)
	tomwidget_original_methods.parent_class.parent_set (p, pp);
    }
  </c>
}

void
  signal_style_set GtkStyle previous_style
{
  pointer p = object, ps;

  if (previous_style != nil)
    ps = [previous_style gtkStyle];

  <c>
    {
      if (tomwidget_original_methods.parent_class.style_set)
	tomwidget_original_methods.parent_class.style_set (p, ps);
    }
  </c>
}

void
  signal_grab_focus
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.grab_focus)
	tomwidget_original_methods.parent_class.grab_focus (p);
    }
  </c>
}

int
  signal_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.event)
	tomwidget_original_methods.parent_class.event (p, e);
    }
  </c>
}

int
  signal_button_press_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.button_press_event)
	tomwidget_original_methods.parent_class.button_press_event (p, e);
    }
  </c>
}

int
  signal_button_release_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.button_release_event)
	tomwidget_original_methods.parent_class.button_release_event (p, e);
    }
  </c>
}

int
  signal_motion_notify_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.motion_notify_event)
	tomwidget_original_methods.parent_class.motion_notify_event (p, e);
    }
  </c>
}

int
  signal_delete_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.delete_event)
	tomwidget_original_methods.parent_class.delete_event (p, e);
    }
  </c>
}

int
  signal_destroy_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.destroy_event)
	tomwidget_original_methods.parent_class.destroy_event (p, e);
    }
  </c>
}

int
  signal_expose_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.expose_event)
	tomwidget_original_methods.parent_class.expose_event (p, e);
    }
  </c>
}

int
  signal_key_press_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.key_press_event)
	tomwidget_original_methods.parent_class.key_press_event (p, e);
    }
  </c>
}

int
  signal_key_release_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.key_release_event)
	tomwidget_original_methods.parent_class.key_release_event (p, e);
    }
  </c>
}

int
  signal_enter_notify_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.enter_notify_event)
	tomwidget_original_methods.parent_class.enter_notify_event (p, e);
    }
  </c>
}

int
  signal_leave_notify_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.leave_notify_event)
	tomwidget_original_methods.parent_class.leave_notify_event (p, e);
    }
  </c>
}

int
  signal_configure_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.configure_event)
	tomwidget_original_methods.parent_class.configure_event (p, e);
    }
  </c>
}

int
  signal_focus_in_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.focus_in_event)
	tomwidget_original_methods.parent_class.focus_in_event (p, e);
    }
  </c>
}

int
  signal_focus_out_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.focus_out_event)
	tomwidget_original_methods.parent_class.focus_out_event (p, e);
    }
  </c>
}

int
  signal_map_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.map_event)
	tomwidget_original_methods.parent_class.map_event (p, e);
    }
  </c>
}

int
  signal_unmap_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.unmap_event)
	tomwidget_original_methods.parent_class.unmap_event (p, e);
    }
  </c>
}

int
  signal_property_notify_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.property_notify_event)
	tomwidget_original_methods.parent_class.property_notify_event (p, e);
    }
  </c>
}

int
  signal_selection_clear_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.selection_clear_event)
	tomwidget_original_methods.parent_class.selection_clear_event (p, e);
    }
  </c>
}

int
  signal_selection_request_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.selection_request_event)
	tomwidget_original_methods.parent_class.selection_request_event (p, e);
    }
  </c>
}

int
  signal_selection_notify_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.selection_notify_event)
	tomwidget_original_methods.parent_class.selection_notify_event (p, e);
    }
  </c>
}

int
  signal_proximity_in_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.proximity_in_event)
	tomwidget_original_methods.parent_class.proximity_in_event (p, e);
    }
  </c>
}

int
  signal_proximity_out_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.proximity_out_event)
	tomwidget_original_methods.parent_class.proximity_out_event (p, e);
    }
  </c>
}

int
  signal_visibility_notify_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.visibility_notify_event)
	tomwidget_original_methods.parent_class.visibility_notify_event (p, e);
    }
  </c>
}

int
  signal_client_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.client_event)
	tomwidget_original_methods.parent_class.client_event (p, e);
    }
  </c>
}

int
  signal_no_expose_event pointer e
{
  pointer p = object;

  <c>
    {
      if (tomwidget_original_methods.parent_class.no_expose_event)
	tomwidget_original_methods.parent_class.no_expose_event (p, e);
    }
  </c>
}

end;
