<copyright> tomgnome CanvasItem class and wrapping subclasses.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter Schoenmakers</a>

    Copyright &copy; 1999 Pieter J. Schoenmakers

    This file is part of tomgnome.  tomgnome is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the tomgtk distribution; see the file COPYING.LIB.

    <id>$Id: CanvasItem.t,v 1.2 1999/04/30 20:46:21 tiggr Exp $</id>
    </copyright>

<gnome_version>1.0.8</gnome_version>

<c>
#include "glue.h"
</c>

/******************** CanvasItem ********************/

implementation class
GnomeCanvasItem: GtkObject

end;

implementation instance
GnomeCanvasItem

<doc> Return the {GnomeCanvasItem} for which this object stands.  </doc>
pointer
  gnomeCanvasItem
{
  = object;
}

/********** object arguments **********/

<doc> These two methods ensure that the canvas items do not need special
    treatment as far as the client code is concerned.  They do what
    gnome_canvas_item_set() et.al. do: request a redraw when visible.
    </doc>

<doc> Request redraw of the current bounding box (before modification).
    (Invoked upon entry to an object argument manipulation function.)
    </doc>
void
  will_set_object_argument
{
  [self redraw_if_visible];
}

<doc> Request redraw of the current bounding box (before modification).
    (Invoked upon exit from an object argument manipulation function.)
    </doc>
void
  did_set_object_argument
{
  [self redraw_if_visible];
  [[self canvas] set_need_repick];
}

/********** object arguments **********/

/********** flags **********/

boolean (v)
  affine_full
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_AFFINE_FULL;
    }
  </c>
}

boolean (v)
  always_redraw
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_ALWAYS_REDRAW;
    }
  </c>
}

boolean (v)
  mapped
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_MAPPED;
    }
  </c>
}

boolean (v)
  need_affine
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_NEED_AFFINE;
    }
  </c>
}

boolean (v)
  need_clip
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_NEED_CLIP;
    }
  </c>
}

boolean (v)
  need_update
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_NEED_UPDATE;
    }
  </c>
}

boolean (v)
  need_vis
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_NEED_VIS;
    }
  </c>
}

boolean (v)
  realized
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_REALIZED;
    }
  </c>
}

boolean (v)
  visible
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      v = item->object.flags & GNOME_CANVAS_ITEM_VISIBLE;
    }
  </c>
}

/********** miscellanea **********/

<doc> Return the {canvas}.  </doc>
GnomeCanvas
  canvas
{
  pointer o, p = object;

  <c>
    {
      GnomeCanvasItem *item = p;
      o = item->canvas;
    }
  </c>

  = [GnomeCanvas proxy o create: YES];
}

<doc> Return the limit points spanning the bounding box of this item.
    </doc>
(double, double, double, double) (x1, y1, x2, y2)
  bounds
{
  pointer p = object;
  <c>
    {
      GnomeCanvasItem *item = p;
      x1 = item->x1;
      y1 = item->y1;
      x2 = item->x2;
      y2 = item->y2;
    }
  </c>
}

<doc> Move the item by the specified amount.  </doc>
void
  move (double, double) (dx, dy)
{
  pointer p = object;
  <c>
    gnome_canvas_item_move (p, dx, dy);
  </c>
}

<doc> Request a redraw of the bounds of this item, if it is visible.
    </doc>
void
  redraw_if_visible
{
  if ([self visible])
    [[self canvas] request_redraw [self bounds]];
} 

<doc> Pass to {gnome_canvas_update_bbox}.  </doc>
void
  update_bbox (int, int, int, int) (x1, y1, x2, y2)
{
  pointer p = object;

  <c>
    gnome_canvas_update_bbox (p, x1, y1, x2, y2);
  </c>
}

end;

/******************** CanvasGroup ********************/

implementation class
GnomeCanvasGroup: GnomeCanvasItem

end;

implementation instance
GnomeCanvasGroup

<doc> Return the {CanvasGroup} for which this object stands.  </doc>
pointer
  gnomeCanvasGroup
{
  = object;
}

void
  child_bounds GnomeCanvasItem item
{
  pointer p = object, o;

  if (item != nil)
    o = [item gnomeCanvasItem];

  <c>
    gnome_canvas_group_child_bounds (p, o);
  </c>
}

/********** object arguments **********/

double
  x
{
  = [self get_double_argument "x"];
}

void
  set_x double x
{
  [self set_double_argument "x" to x];
}

double
  y
{
  = [self get_double_argument "y"];
}

void
  set_y double y
{
  [self set_double_argument "y" to y];
}

end;

/******************** CanvasRE ********************/

implementation class
GnomeCanvasRE: GnomeCanvasItem

end;

implementation instance
GnomeCanvasRE

<doc> Return the {GnomeCanvasRE} for which this object stands.  </doc>
pointer
  gnomeCanvasRE
{
  = object;
}

/********** object arguments **********/

double
  x1
{
  = [self get_double_argument "x1"];
}

void
  set_x1 double x1
{
  [self set_double_argument "x1" to x1];
}

double
  y1
{
  = [self get_double_argument "y1"];
}

void
  set_y1 double y1
{
  [self set_double_argument "y1" to y1];
}

double
  x2
{
  = [self get_double_argument "x2"];
}

void
  set_x2 double x2
{
  [self set_double_argument "x2" to x2];
}

double
  y2
{
  = [self get_double_argument "y2"];
}

void
  set_y2 double y2
{
  [self set_double_argument "y2" to y2];
}

void
  set_fill_color String t
{
  [self set_string_argument "fill_color" to t];
}

// fill_color_gdk RW

void
  set_outline_color String t
{
  [self set_string_argument "outline_color" to t];
}

// outline_color_gdk RW
// fill_stipple RW
// outline_stipple RW

int
  width_pixels
{
  = [self get_int_argument "width_pixels"];
}

void
  set_width_pixels int u
{
  [self set_int_argument "width_pixels" to u];
}

double
  width_units
{
  = [self get_double_argument "width_units"];
}

void
  set_width_units double u
{
  [self set_double_argument "width_units" to u];
}

end;

/******************** CanvasRect ********************/

implementation class
GnomeCanvasRect: GnomeCanvasRE

end;

implementation instance
GnomeCanvasRect

<doc> Return the {CanvasRect} for which this object stands.  </doc>
pointer
  gnomeCanvasRect
{
  = object;
}

end;

/******************** CanvasEllipse ********************/

implementation class
GnomeCanvasEllipse: GnomeCanvasRE

end;

implementation instance
GnomeCanvasEllipse

<doc> Return the {GnomeCanvasEllipse} for which this object stands.  </doc>
pointer
  gnomeCanvasEllipse
{
  = object;
}

end;

/******************** CanvasText ********************/

implementation class
GnomeCanvasText: GnomeCanvasItem

end;

implementation instance
GnomeCanvasText

<doc> Return the {GnomeCanvasText} for which this object stands.  </doc>
pointer
  gnomeCanvasText
{
  = object;
}

/********** object arguments **********/

String
  text
{
  = [self get_string_argument "text"];
}

void
  set_text String t
{
  [self set_string_argument "text" to t];
}

double
  x
{
  = [self get_double_argument "x"];
}

void
  set_x double x
{
  [self set_double_argument "x" to x];
}

double
  y
{
  = [self get_double_argument "y"];
}

void
  set_y double y
{
  [self set_double_argument "y" to y];
}

void
  set_font String t
{
  [self set_string_argument "font" to t];
}

void
  set_fontset String t
{
  [self set_string_argument "fontset" to t];
}

// font_gdk RW

int
  anchor
{
  = [self get_int_argument "anchor"];
}

void
  set_anchor int GtkAnchorType_something
{
  [self set_int_argument "anchor" to GtkAnchorType_something];
}

int
  justification
{
  = [self get_int_argument "justification"];
}

void
  set_justification int GtkJustification_something
{
  [self set_int_argument "justification" to GtkJustification_something];
}

void
  set_fill_color String t
{
  [self set_string_argument "fill_color" to t];
}

// fill_color_gdk RW
// fill_stipple RW

double
  clip_width
{
  = [self get_double_argument "clip_width"];
}

void
  set_clip_width double clip_width
{
  [self set_double_argument "clip_width" to clip_width];
}

double
  clip_height
{
  = [self get_double_argument "clip_height"];
}

void
  set_clip_height double clip_height
{
  [self set_double_argument "clip_height" to clip_height];
}

boolean
  clip
{
  = [self get_boolean_argument "clip"];
}

void
  set_clip boolean clip
{
  [self set_boolean_argument "clip" to clip];
}

double
  x_offset
{
  = [self get_double_argument "x_offset"];
}

void
  set_x_offset double x_offset
{
  [self set_double_argument "x_offset" to x_offset];
}

double
  y_offset
{
  = [self get_double_argument "y_offset"];
}

void
  set_y_offset double y_offset
{
  [self set_double_argument "y_offset" to y_offset];
}

double
  text_width
{
  = [self get_double_argument "text_width"];
}

double
  text_height
{
  = [self get_double_argument "text_height"];
}

end;
