<copyright> tomgnome Client class.
    Written by <a href="mailto:yacc@gerbil.org">Andreas Kostyrka</a>

    Copyright &copy; 1998 Andreas Kostyrka

    This file is part of tomgnome.  tomgnome is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the tomgtk distribution; see the file COPYING.LIB.

    <id>$Id: Client.t,v 1.4 1999/04/30 20:46:23 tiggr Exp $</id>
    </copyright>

<gtk_version>1.1.0</gtk_version>

<c>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <tom/util.h>
#include "tomutil.h"
</c>

implementation class
GnomeClient : GtkObject

<doc>
   Create a new session management client and try to connect to a
   session manager. 
</doc>

instance (id)
  new
{
  pointer client;
  <c> client = gnome_client_new (); </c>
  = [[self alloc] init client];
}

<doc> Create a new session management client. </doc>

instance (id)
  new_without_connection
{
  pointer client;
  <c> client = gnome_client_new_without_connection (); </c>
  = [[self alloc] init client];
}

end;

implementation instance
GnomeClient

<doc> Return the {GnomeClient} for which this object stands.  </doc>
pointer
  gnomeClient
{
  = object;
}

<doc>
   Try to connect to a session manager.  If the client was created
   with a valid session management client id, we will try to connect
   to the manager with this old id.  If the connection was successfull,
   the 'connect' signal will be emitted, after some default properties
   have been sent to the session manager.
</doc>
void
  connect
{
  pointer obj = object;
  <c> gnome_client_connect ((GnomeClient *) obj); </c>
}

<doc>
   Disconnect from the session manager.  After disconnecting, the
   'disconnect' signal will be emitted.
</doc>

void
  disconnect
{
  pointer obj = object;
  <c> gnome_client_disconnect ((GnomeClient *) obj); </c>
}

<doc>
   Set the client id.  This is only possible, if the client is not
   connected to a session manager.
</doc>

void
  set_id String client_id
{
  pointer obj = object;
  <c> {
    int helplen;
    char *ccid;

    C_STRING_WITH_TOM_STRING (ccid, helplen, client_id);
    gnome_client_set_id ((GnomeClient *) obj, ccid);
  } </c>
}

<doc>
   Get the client id of a session management client object.  If this
   object has never been connected to a session manager and a client
   id has not been set, this function returns nil.
</doc>
String (res)
  get_id
{
  pointer obj = object;

  <c> {
    char *cres;
    cres = gnome_client_get_id ((GnomeClient *) obj);
    if (cres)
      res = (void *) byte_string_with_c_string (cres);
  } </c>
}

<doc>
   Get the client id from the last session.  If this object was not
   recreated from a previous session, returns nil.  This is useful
   because a client might choose to store configuration information
   in a location based on the session id; this old session id must be
   recalled later to find the information.
</doc>
String (res)
  get_previous_id
{
  pointer obj = object;

  <c> {
    char *cres;
    cres = gnome_client_get_previous_id ((GnomeClient *) obj);
    if (cres)
      res = (void *) byte_string_with_c_string (cres);
  } </c>
}

<doc>
   The follwing functions are used to set or unset some session
   management properties.  

   To unset a property, you have to specify a nil
   value.
   You are not allowed to unset the properties
   marked as required.  There is one exception to this
   rule: If you unset the clone command, the clone command will be set
   to the value of the restart command.

   The magic "--sm-client-id" option is automatically appended to the
   restart command.  Do not include this option in the restart command
   that you set. 
</doc>

void
  set_clone_commend Array args
{
  pointer obj = object;
  
  if (args == nil) {
    <c> gnome_client_set_clone_command ((GnomeClient *) obj, 0, NULL); </c>
  } else {
    int alen = [args length];
    int helplen;
    int i;
    String onearg;
    
    <c> {
      char **argv = alloca ((alen+1) * sizeof *argv);
      
      for (i = 0; i < alen; i++) {
	</c> onearg = args[i]; <c>
	C_STRING_WITH_TOM_STRING (argv[i], helplen, onearg);
      }
      gnome_client_set_clone_command ((GnomeClient *) obj, alen, argv);
    } </c>
  }
}

void
  set_current_directory String dir
{
  pointer obj = object;
  if (dir == nil) {
    <c> gnome_client_set_current_directory ((GnomeClient *) obj, NULL); </c>
  } else {
    <c> {
      int len;
      char *cdir;
    
      C_STRING_WITH_TOM_STRING (cdir, len, dir);
      gnome_client_set_current_directory ((GnomeClient *) obj, cdir);
    } </c>
  }
}

void
  set_process_id int pid
{
  pointer obj = object;
  <c> gnome_client_set_process_id ((GnomeClient *) obj, pid); </c>
}

void
  set_discard_command Array args
{
  pointer obj = object;
  if (args == nil) {
    <c> gnome_client_set_discard_command ((GnomeClient *) obj, 0, NULL); </c>
  } else {
    <c> {
      int argc;
      char **argv;
      
      argv = tomarray_to_charptrarray ((tom_object) args, &argc);
      gnome_client_set_discard_command ((GnomeClient *) obj, argc, argv);
      free_charptrarray (argv);
    } </c>
  }
}

void
  set_environment Array args
{
  [self unimplemented cmd];
//  pointer obj = object;
//  if (args == nil) {
//    <c> gnome_client_set_environment ((GnomeClient *) obj, 0, NULL); </c>
//  } else {
//    <c> {
//      int argc;
//      char **argv;
//      
//      argv = tomarray_to_charptrarray ((tom_object) args, &argc);
//      gnome_client_set_environment ((GnomeClient *) obj, argc, argv);
//      free_charptrarray (argv);
//    } </c>
//  }
}

void
  set_program String program
{
  pointer obj = object;
  if (program == nil) {
    <c> gnome_client_set_program ((GnomeClient *) obj, NULL); </c>
  } else {
    <c> {
      int len;
      char *cprogram;
    
      C_STRING_WITH_TOM_STRING (cprogram, len, program);
      gnome_client_set_program ((GnomeClient *) obj, cprogram);
    } </c>
  }
}

void
  set_restart_command Array args
{
  pointer obj = object;
  if (args == nil) {
    <c> gnome_client_set_restart_command ((GnomeClient *) obj, 0, NULL); </c>
  } else {
    <c> {
      int argc;
      char **argv;
      
      argv = tomarray_to_charptrarray ((tom_object) args, &argc);
      gnome_client_set_restart_command ((GnomeClient *) obj, argc, argv);
      free_charptrarray (argv);
    } </c>
  }
}

void
  set_resign_command Array args
{
  pointer obj = object;
  if (args == nil) {
    <c> gnome_client_set_resign_command ((GnomeClient *) obj, 0, NULL); </c>
  } else {
    <c> {
      int argc;
      char **argv;
      
      argv = tomarray_to_charptrarray ((tom_object) args, &argc);
      gnome_client_set_resign_command ((GnomeClient *) obj, argc, argv);
      free_charptrarray (argv);
    } </c>
  }
}

void
  set_shutdown_command Array args
{
  pointer obj = object;
  if (args == nil) {
    <c> gnome_client_set_shutdown_command ((GnomeClient *) obj, 0, NULL); </c>
  } else {
    <c> {
      int argc;
      char **argv;
      
      argv = tomarray_to_charptrarray ((tom_object) args, &argc);
      gnome_client_set_shutdown_command ((GnomeClient *) obj, argc, argv);
      free_charptrarray (argv);
    } </c>
  }
}

void
  set_restart_style int GnomeRestartStyle_style
{
  pointer obj = object;
  <c> gnome_client_set_restart_style ((GnomeClient *) obj, 
				      GnomeRestartStyle_style); </c>
}

void
  set_user_id String uid
{
  pointer obj = object;
  if (uid == nil) {
    <c> gnome_client_set_user_id ((GnomeClient *) obj, NULL); </c>
  } else {
    <c> {
      int len;
      char *cuid;
    
      C_STRING_WITH_TOM_STRING (cuid, len, uid);
      gnome_client_set_user_id ((GnomeClient *) obj, cuid);
    } </c>
  }
}

<doc>
   The following function can be used, if you want that the
   'save_yourself' signal is emitted again, after all clients finished
   saveing.
</doc>
void
  request_phase_2
{
  pointer obj = object;
  <c> gnome_client_request_phase_2 ((GnomeClient *) obj); </c>
}

end;

<comment>

/* Use the following functions, if you want to interact with the user
   will saveing his data.  If the session manager decides that it's
   out turn to interact, 'func' will be called with the specified
   arguments and a special 'GnomeInteractionKey'.  The session manager
   will block other clients from interacting until this key is
   returned with 'gnome_interaction_key_return'.  */

void         gnome_client_request_interaction    (GnomeClient *client,
						  GnomeDialogType dialog,
						  GnomeInteractFunction func,
						  gpointer client_data);

void         gnome_client_request_interaction_interp (GnomeClient *client,
						      GnomeDialogType dialog,
						      GtkCallbackMarshal func,
						      gpointer data,
						      GtkDestroyNotify destroy);

/* 'gnome_interaction_key_return' is used to tell gnome, that you are
   finished with interaction */
void         gnome_interaction_key_return        (gint key,
						  gint cancel_shutdown);

/* Request the session manager to save the session in some way.  This
   can also be used to request a logout.  If IS_GLOBAL is true, then
   all clients are saved; otherwise only this client is saved.  */
void         gnome_client_request_save (GnomeClient        *client,
					GnomeSaveStyle      save_style,
					int /* bool */      shutdown,
					GnomeInteractStyle  interact_style,
					int /* bool */      fast,
					int /* bool */      global);

/* This will force the underlying connection to the session manager to
   be flushed.  This is useful if you have some pending changes that
   you want to make sure get committed.  */
void         gnome_client_flush (GnomeClient *client);

</comment>

