<copyright> tomgnome TOMCanvasItem class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter Schoenmakers</a>

    Copyright &copy; 1999 Pieter J. Schoenmakers

    This file is part of tomgnome.  tomgnome is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the tomgtk distribution; see the file COPYING.LIB.

    <id>$Id: TOMCanvasItem.t,v 1.3 1999/04/30 20:46:36 tiggr Exp $</id>
    </copyright>

<gnome_version>1.0.8</gnome_version>

<c>
#include "glue.h"
#include "tom-canvas-item.h"

static GnomeCanvasItemClass *parent_class = NULL;
</c>

/******************** TOMCanvasItem ********************/

<doc> The {TOMCanvasItem} class is the superclass for canvas items that
    are written in TOM.

    To the Gtk machinery, all {TOMCanvasItem} subclasses will appear to be
    of the same Gtk class (named GnomeTomCanvasItem).  Furthermore, the
    {TOMCanvasItem} subclasses can not define addition signals; only those
    inherited from {GnomeCanvasItem} are available.

    See also the {TOMWidget} class.  </doc>
implementation class
TOMCanvasItem: GnomeCanvasItem

end;

implementation instance
TOMCanvasItem

<doc> Designated initializer.  </doc>
id
  initInGroup GnomeCanvasGroup group
{
  pointer p, parent = [group gnomeCanvasGroup];

  <c>
    p = gnome_tom_canvas_item_new (parent, self);
  </c>

  if (p != object)
    [self unimplemented cmd
	  message: "TOMCanvasItem object creation mismatch"];

  = self;
}

/******************** GtkObject slots ********************/

void
  signal_set_arg (pointer, int) (arg, arg_id)
{
  pointer p = object;

  <c>
    if (parent_class->parent_class.set_arg)
      parent_class->parent_class.set_arg (p, arg, arg_id);
  </c>
}

void
  signal_get_arg (pointer, int) (arg, arg_id)
{
  pointer p = object;

  <c>
    if (parent_class->parent_class.get_arg)
      parent_class->parent_class.get_arg (p, arg, arg_id);
  </c>
}

void
  signal_shutdown
{
  pointer p = object;

  <c>
    if (parent_class->parent_class.shutdown)
      parent_class->parent_class.shutdown (p);
  </c>
}

void
  signal_destroy
{
  pointer p = object;

  <c>
    if (parent_class->parent_class.destroy)
      parent_class->parent_class.destroy (p);
  </c>
}

void
  signal_finalize
{
  pointer p = object;

  <c>
    if (parent_class->parent_class.finalize)
      parent_class->parent_class.finalize (p);
  </c>
}

/******************** GnomeCanvasItem slots ********************/

void
  signal_item_update (pointer, pointer, int) (affine, clip_path, flags)
{
  pointer p = object;

  <c>
    if (parent_class->update)
      parent_class->update (p, affine, clip_path, flags);
  </c>
}

void
  signal_item_realize
{
  pointer p = object;

  <c>
    if (parent_class->realize)
      parent_class->realize (p);
  </c>
}

void
  signal_item_unrealize
{
  pointer p = object;

  <c>
    if (parent_class->unrealize)
      parent_class->unrealize (p);
  </c>
}

void
  signal_item_map
{
  pointer p = object;

  <c>
    if (parent_class->map)
      parent_class->map (p);
  </c>
}

void
  signal_item_unmap
{
  pointer p = object;

  <c>
    if (parent_class->unmap)
      parent_class->unmap (p);
  </c>
}

pointer
  signal_item_coverage
{
  pointer p = object;

  <c>
    if (parent_class->coverage)
      parent_class->coverage (p);
  </c>
}

void
  signal_item_draw (pointer, int, int, int, int) (drawable, x, y, width, height)
{
  pointer p = object;

  <c>
    if (parent_class->draw)
      parent_class->draw (p, drawable, x, y, width, height);
  </c>
}

void
  signal_item_render pointer buf
{
  pointer p = object;

  <c>
    if (parent_class->render)
      parent_class->render (p, buf);
  </c>
}

(double, pointer) (distance, actual_item)
  signal_item_point (double, double, int, int) (x, y, cx, cy)
{
  pointer p = object;

  <c>
    if (parent_class->point)
      distance = parent_class->point (p, x, y, cx, cy, actual_item);
  </c>
}

void
  signal_item_translate (double, double) (dx, dy)
{
  pointer p = object;

  <c>
    if (parent_class->translate)
      parent_class->translate (p, dx, dy);
  </c>
}

(double, double, double, double) (x1, y1, x2, y2)
  signal_item_bounds
{
  pointer p = object;

  <c>
    if (parent_class->bounds)
      {
	double cx1, cx2, cy1, cy2;
	parent_class->bounds (p, &cx1, &cy1, &cx2, &cy2);
	x1 = cx1, x2 = cx2, y1 = cy1, y2 = cy2;
      }
  </c>
}

int (result)
  signal_item_event pointer event
{
  pointer p = object;

  <c>
    if (parent_class->event)
      result = parent_class->event (p, event);
  </c>
}

end;

/* The administration for Gtk.  */
<c>
static void
gnome_tom_canvas_item_class_init (GnomeTomCanvasItemClass *klass)
{
  GtkObjectClass *object_class = (GtkObjectClass*) klass;
  GnomeCanvasItemClass *canvas_item_class = (GnomeCanvasItemClass*) klass;

  parent_class = gtk_type_class (GNOME_TYPE_CANVAS_ITEM);

  object_class->set_arg = tomcanvasitem_object_set_arg;
  object_class->get_arg = tomcanvasitem_object_get_arg;
  object_class->shutdown = tomcanvasitem_object_shutdown;
  object_class->destroy = tomcanvasitem_object_destroy;
  object_class->finalize = tomcanvasitem_object_finalize;

  canvas_item_class->update = tomcanvasitem_canvas_item_update;
  canvas_item_class->realize = tomcanvasitem_canvas_item_realize;
  canvas_item_class->unrealize = tomcanvasitem_canvas_item_unrealize;
  canvas_item_class->map = tomcanvasitem_canvas_item_map;
  canvas_item_class->unmap = tomcanvasitem_canvas_item_unmap;
  canvas_item_class->coverage = tomcanvasitem_canvas_item_coverage;
  canvas_item_class->draw = tomcanvasitem_canvas_item_draw;
  canvas_item_class->render = tomcanvasitem_canvas_item_render;
  canvas_item_class->point = tomcanvasitem_canvas_item_point;
  canvas_item_class->translate = tomcanvasitem_canvas_item_translate;
  canvas_item_class->bounds = tomcanvasitem_canvas_item_bounds;
  canvas_item_class->event = tomcanvasitem_canvas_item_event;
}

static void
gnome_tom_canvas_item_init (GnomeTomCanvasItem *w, void *real_class)
{
  /* No need to do anything.  */
}

GtkType
gnome_tom_canvas_item_get_type (void)
{
  static GtkType tom_canvas_item_type = 0;

  if (!tom_canvas_item_type)
    {
      static const GtkTypeInfo tom_canvas_item_info =
      {
	"TOMCanvasItem",
	sizeof (GnomeTomCanvasItem),
	sizeof (GnomeTomCanvasItemClass),
	(GtkClassInitFunc) gnome_tom_canvas_item_class_init,
	(GtkObjectInitFunc) gnome_tom_canvas_item_init,
        /* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      tom_canvas_item_type = gtk_type_unique (GNOME_TYPE_CANVAS_ITEM,
					      &tom_canvas_item_info);
    }

  return tom_canvas_item_type;
}

GnomeCanvasItem *
gnome_tom_canvas_item_new (GnomeCanvasGroup *parent, void *proxy)
{
  tomcanvasitem_created = proxy;
  return gnome_canvas_item_new (parent, gnome_tom_canvas_item_get_type (),
				NULL);
}

</c>
